@@ -19,8 +19,8 @@ local sysdetect = require("luarocks.core.sysdetect")
 
 --------------------------------------------------------------------------------
 
-local program_version = "3.3.1"
-local program_series = "3.3"
+local program_version = "3.4.0"
+local program_series = "3.4"
 local major_version = (program_version:match("([^.]%.[^.])")) or program_series
 
 local is_windows = package.config:sub(1,1) == "\\"
@@ -206,7 +206,7 @@ local function make_defaults(lua_version, target_cpu, platforms, home)
          {
            "https://luarocks.org",
            "https://raw.githubusercontent.com/rocks-moonscript-org/moonrocks-mirror/master/",
-           "http://luafr.org/moonrocks/",
+           "https://luafr.org/luarocks/",
            "http://luarocks.logiceditor.com/rocks",
          }
       },
@@ -504,6 +504,19 @@ local function use_defaults(cfg, defaults)
    end
 end
 
+local function get_first_arg()
+   if not arg then
+      return
+   end
+   local first_arg = arg[0]
+   local i = -1
+   while arg[i] do
+      first_arg = arg[i]
+      i = i -1
+   end
+   return first_arg
+end
+
 --------------------------------------------------------------------------------
 
 local cfg = {}
@@ -542,14 +555,20 @@ function cfg.init(detected, warning)
    cfg.program_series = program_series
    cfg.major_version = major_version
 
+   if hardcoded.IS_BINARY then
+      cfg.is_binary = true
+   end
+
    -- Use detected values as defaults, overridable via config files or CLI args
+   
+   local first_arg = get_first_arg()
 
    cfg.lua_version = detected.lua_version or hardcoded.LUA_VERSION or _VERSION:sub(5)
-   cfg.lua_interpreter = detected.lua_interpreter or hardcoded.LUA_INTERPRETER or (arg and arg[-1] and arg[-1]:gsub(".*[\\/]", "")) or (is_windows and "lua.exe" or "lua")
+   cfg.lua_interpreter = detected.lua_interpreter or hardcoded.LUA_INTERPRETER or (first_arg and first_arg:gsub(".*[\\/]", "")) or (is_windows and "lua.exe" or "lua")
    cfg.project_dir = (not hardcoded.FORCE_CONFIG) and detected.project_dir
 
    do
-      local lua_bindir = detected.lua_bindir or hardcoded.LUA_BINDIR or (arg and arg[-1] and arg[-1]:gsub("[\\/][^\\/]+$", ""))
+      local lua_bindir = detected.lua_bindir or hardcoded.LUA_BINDIR or (first_arg and first_arg:gsub("[\\/][^\\/]+$", ""))
       local lua_dir = detected.lua_dir or hardcoded.LUA_DIR or (lua_bindir and lua_bindir:gsub("[\\/]bin$", ""))
       cfg.variables = {
          LUA_DIR = lua_dir,
@@ -671,6 +690,8 @@ function cfg.init(detected, warning)
    if detected.given_lua_dir then
       cfg.variables.LUA_DIR = detected.given_lua_dir
       cfg.variables.LUA_BINDIR = detected.lua_bindir
+      cfg.variables.LUA_LIBDIR = nil
+      cfg.variables.LUA_INCDIR = nil
       cfg.lua_interpreter = detected.lua_interpreter
    end
 
