@@ -534,6 +534,8 @@ local function check_external_dependency(name, ext_files, vars, mode, cache)
    local prefixes
    if vars[name .. "_DIR"] then
       prefixes = { vars[name .. "_DIR"] }
+   elseif vars.DEPS_DIR then
+      prefixes = { vars.DEPS_DIR }
    else
       prefixes = cfg.external_deps_dirs
    end
@@ -649,6 +651,23 @@ function deps.scan_deps(results, manifest, name, version, deps_mode)
    end
 end
 
+local function lua_h_exists(d, luaver)
+   local n = tonumber(luaver)
+   local major = math.floor(n)
+   local minor = (n - major) * 10
+   local luanum = math.floor(major * 100 + minor)
+
+   local lua_h = dir.path(d, "lua.h")
+   local fd = io.open(lua_h)
+   if fd then
+      local data = fd:read("*a")
+      fd:close()
+      if data:match("LUA_VERSION_NUM%s*" .. tostring(luanum)) then
+         return d
+      end
+   end
+end
+
 local function find_lua_incdir(prefix, luaver, luajitver)
    luajitver = luajitver and luajitver:gsub("%-.*", "")
    local shortv = luaver:gsub("%.", "")
@@ -662,11 +681,7 @@ local function find_lua_incdir(prefix, luaver, luajitver)
       luajitver and prefix .. "/include/luajit-" .. luajitver:match("^(%d+%.%d+)"),
    }
    for _, d in ipairs(incdirs) do
-      local lua_h = dir.path(d, "lua.h")
-      local fd = io.open(lua_h)
-      if fd then
-         -- TODO check that LUA_VERSION_MAJOR and LUA_VERSION_MINOR match luaver
-         fd:close()
+      if lua_h_exists(d, luaver) then
          return d
       end
    end
@@ -678,14 +693,18 @@ end
 function deps.check_lua_incdir(vars)
    local ljv = util.get_luajit_version()
 
-   if (not vars.LUA_INCDIR) and vars.LUA_DIR then
+   if vars.LUA_INCDIR then
+      return lua_h_exists(vars.LUA_INCDIR, cfg.lua_version)
+   end
+   
+   if vars.LUA_DIR then
       vars.LUA_INCDIR = find_lua_incdir(vars.LUA_DIR, cfg.lua_version, ljv)
-      if vars.LUA_INCDIR == nil then
-         return nil, "Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.", "dependency"
+      if vars.LUA_INCDIR then
+         return true
       end
    end
 
-   return true
+   return nil, "Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.", "dependency"
 end
 
 function deps.check_lua_libdir(vars)
@@ -703,7 +722,9 @@ function deps.check_lua_libdir(vars)
       table.insert(libnames, 1, "luajit-" .. cfg.lua_version)
    end
    local cache = {}
+   local save_LUA_INCDIR = vars.LUA_INCDIR
    local ok = check_external_dependency("LUA", { library = libnames }, vars, "build", cache)
+   vars.LUA_INCDIR = save_LUA_INCDIR
    if ok then
       vars.LUALIB = vars.LUA_LIBDIR_FILE
       return true
